package com.ruoyi.bst.device.service.impl;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.bst.device.domain.Device;
import com.ruoyi.bst.device.domain.DeviceQuery;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.domain.enums.DeviceLockStatus;
import com.ruoyi.bst.device.domain.enums.DeviceStatus;
import com.ruoyi.bst.device.mapper.DeviceMapper;
import com.ruoyi.bst.device.service.DeviceIotService;
import com.ruoyi.bst.device.utils.DeviceUtil;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.response.CommandResponse;
import com.ruoyi.iot.enums.IotHttpStatus;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.iot.service.impl.DeviceOnlineStatus;
import com.ruoyi.iot.util.IotUtil;

import lombok.extern.slf4j.Slf4j;

@Service
@Slf4j
public class DeviceIotServiceImpl implements DeviceIotService {

    @Autowired
    private IotService iotService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Override
    public int unlock(DeviceVO device, boolean isAdmin, String reason) {
        if (device == null || device.getId() == null) {
            return 0;
        }

        if (isAdmin) {
            ServiceUtil.assertion(!DeviceStatus.canAdminUnlock().contains(device.getStatus()), "设备%s当前状态不允许管理员开锁", device.getSn());
        } else {
            ServiceUtil.assertion(!DeviceStatus.canUserUnlock().contains(device.getStatus()), "设备%s当前状态不允许用户开锁", device.getSn());
        }

        Integer result = transactionTemplate.execute(status -> {
            // 更新设备状态
            Device data = new Device();
            data.setLockStatus(DeviceLockStatus.OPEN.getCode());
            DeviceQuery query = new DeviceQuery();
            query.setId(device.getId());
            // 管理员开锁
            if (isAdmin) {
                data.setStatus(DeviceStatus.DISPATCHING.getCode());
                query.setStatusList(DeviceStatus.canAdminUnlock());
            }
            // 用户开锁
            else {
                data.setStatus(DeviceStatus.IN_USE.getCode());
                query.setStatusList(DeviceStatus.canUserUnlock());
            }
            int rows = deviceMapper.updateByQuery(data, query);

            if (rows > 0) {
                // 发送命令开锁
                CommandResponse res = iotService.unlock(device, reason);
                ServiceUtil.assertion(!IotUtil.isSuccess(res), IotUtil.getMsg(res));
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int lock(DeviceVO device, boolean isAdmin, String reason) {
        if (device == null || device.getId() == null) {
            return 0;
        }

        ServiceUtil.assertion(!DeviceStatus.canLock().contains(device.getStatus()), "设备%s当前状态不允许锁车", device.getSn());

        // TODO 是否有正在进行的订单，需要根据实际情况判断
        boolean hasOrder = false;

        Integer result = transactionTemplate.execute(status -> {
            // 更新设备状态
            Device data = new Device();
            data.setLockStatus(DeviceLockStatus.CLOSE.getCode());
            data.setStatus(hasOrder ? DeviceStatus.TEMP_LOCKED.getCode() : DeviceStatus.AVAILABLE.getCode());

            DeviceQuery query = new DeviceQuery();
            query.setId(device.getId());
            query.setStatusList(DeviceStatus.canLock());
            int rows = deviceMapper.updateByQuery(data, query);

            if (rows > 0) {
                // 发送命令锁车
                CommandResponse res = iotService.lock(device, 3600, reason);
                ServiceUtil.assertion(!IotUtil.isSuccess(res), IotUtil.getMsg(res));
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    @Override
    public boolean play(DeviceVO device, String type, String reason) {
        if (device == null) {
            return false;
        }
        CommandResponse res = iotService.play(device, type, reason);
        return IotUtil.isSuccess(res);
    }

    @Override
    public boolean reboot(DeviceVO device, String reason) {
        if (device == null) {
            return false;
        }
        CommandResponse res = iotService.reboot(device, reason);
        return IotUtil.isSuccess(res);
    }

    @Override
    public boolean unlockSeat(DeviceVO device, String reason) {
        if (device == null) {
            return false;
        }
        CommandResponse res = iotService.unlockSeat(device, reason);
        return IotUtil.isSuccess(res);
    }

    @Override
    public void refresh(List<DeviceVO> deviceList, String onlineType) {
        if (CollectionUtils.isEmptyElement(deviceList)) {
            return;
        }

        // 获取物联网信息
        List<IotDeviceInfo> iotList = iotService.getDeviceInfo(deviceList);
        if (CollectionUtils.isEmptyElement(iotList)) {
            return;
        }

        // 更新设备信息
        for (DeviceVO device : deviceList) {
            if (device == null || StringUtils.isBlank(device.getMac())) {
                continue;
            }
            IotDeviceInfo iot = iotList.stream()
                    .filter(item -> Objects.equals(item.getMac(), device.getMac()))
                    .findFirst().orElse(null);
            DeviceUtil.setIotInfo(device, iot);
        }

        // 异步更新数据库
        scheduledExecutorService.schedule(() -> {
            for (DeviceVO device : deviceList) {
                try {
                    // 获取在线状态
                    if (StringUtils.hasText(onlineType)) {
                        String onlineStatus = iotService.getOnlineStatus(device, onlineType);
                        this.setOnlineStatus(device, onlineStatus);
                    }
                    // 更新物联网信息
                    this.updateIot(device);
                } catch (Exception e) {
                    log.error("更新设备{}物联网信息失败", device.getSn(), e);
                }
            }
        }, 0, TimeUnit.SECONDS);
    }

    private void setOnlineStatus(DeviceVO device, String onlineStatus) {
        // 是否在线
        device.setOnlineStatus(onlineStatus);
        if (DeviceOnlineStatus.ONLINE.getStatus().equals(onlineStatus)) {
            device.setLastOnlineTime(LocalDateTime.now());
        }
    }

    public int updateIot(Device device) {
        if (device == null || device.getId() == null) {
            return 0;
        }
        Device data = new Device();
        data.setId(device.getId());
        data.setVoltage(device.getVoltage());
        data.setSignalStrength(device.getSignalStrength());
        data.setQuality(device.getQuality());
        data.setSatellites(device.getSatellites());
        data.setIotStatus(device.getIotStatus());
        data.setLongitude(device.getLongitude());
        data.setLatitude(device.getLatitude());
        data.setRemainingPower(device.getRemainingPower());
        data.setLastTime(device.getLastTime());
        data.setLastLocationTime(device.getLastLocationTime());
        data.setOnlineStatus(device.getOnlineStatus());
        data.setLastOnlineTime(device.getLastOnlineTime());
        return deviceMapper.updateDevice(data);
    }

    @Override
    public boolean create(String mac) {
        if (StringUtils.isBlank(mac)) {
            return false;
        }
        int code = iotService.create(mac);
        IotHttpStatus status = IotHttpStatus.convertByCode(code);
        ServiceUtil.assertion(!status.equals(IotHttpStatus.SUCCESS) && !status.equals(IotHttpStatus.DEVICE_EXIST),
             "设备【%s】注册失败:%s", mac, status.getMsg());
        return true;
    }

}
