package com.ruoyi.bst.device.service.impl;

import java.util.Collections;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.github.pagehelper.PageHelper;
import com.ruoyi.bst.device.domain.Device;
import com.ruoyi.bst.device.domain.DeviceQuery;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.domain.enums.DeviceStatus;
import com.ruoyi.bst.device.mapper.DeviceMapper;
import com.ruoyi.bst.device.service.DeviceIotService;
import com.ruoyi.bst.device.service.DeviceService;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;

import lombok.extern.slf4j.Slf4j;

/**
 * 设备Service业务层处理
 *
 * @author ruoyi
 * @date 2025-03-20
 */
@Service
@Slf4j
public class DeviceServiceImpl implements DeviceService
{
    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private DeviceIotService deviceIotService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询设备
     *
     * @param id 设备主键
     * @return 设备
     */
    @Override
    public DeviceVO selectDeviceById(Long id, boolean scope)
    {
        DeviceQuery query = new DeviceQuery();
        query.setId(id);
        query.setScope(scope);
        return this.selectOne(query);
    }

    @Override
    public DeviceVO selectOne(DeviceQuery query) {
        PageHelper.startPage(1, 1);
        List<DeviceVO> list = this.selectDeviceList(query);
        return CollectionUtils.firstElement(list);
    }

    /**
     * 查询设备列表
     *
     * @param device 设备
     * @return 设备
     */
    @Override
    public List<DeviceVO> selectDeviceList(DeviceQuery device)
    {
        return deviceMapper.selectDeviceList(device);
    }

    /**
     * 新增设备
     *
     * @param device 设备
     * @return 结果
     */
    @Override
    public int insertDevice(Device device)
    {
        device.setCreateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            int row = deviceMapper.insertDevice(device);

            if (row > 0) {
                // 注册OneNet设备
                deviceIotService.create(device.getMac());
            }
            return row;
        });

        return result == null ? 0 : result;
    }

    /**
     * 修改设备
     *
     * @param device 设备
     * @return 结果
     */
    @Override
    public int updateDevice(Device device)
    {
        device.setUpdateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            int row = deviceMapper.updateDevice(device);
            if (row > 0 && StringUtils.hasText(device.getMac())) {
                // 注册OneNet设备
                deviceIotService.create(device.getMac());
            }
            return row;
        });
        return result == null ? 0 : result;
    }

    /**
     * 批量删除设备
     *
     * @param ids 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteDeviceByIds(Long[] ids)
    {
        return deviceMapper.deleteDeviceByIds(ids);
    }

    /**
     * 删除设备信息
     *
     * @param id 设备主键
     * @return 结果
     */
    @Override
    public int deleteDeviceById(Long id)
    {
        return deviceMapper.deleteDeviceById(id);
    }

    @Override
    public int logicDel(List<Long> ids) {
        return deviceMapper.logicDel(ids);
    }

    @Override
    public List<DeviceVO> selectDeviceByIds(List<Long> ids, boolean scope) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        DeviceQuery query = new DeviceQuery();
        query.setIds(ids);
        query.setScope(scope);
        return deviceMapper.selectDeviceList(query);
    }

    @Override
    public int in(List<DeviceVO> deviceList) {
        if (CollectionUtils.isEmptyElement(deviceList)) {
            return 0;
        }
        int rows = 0;
        for (DeviceVO device : deviceList) {
            if (device == null) {
                continue;
            }
            try {
                int row = this.in(device);
                ServiceUtil.assertion(row != 1, "设备%s入仓失败，请刷新后重试", device.getSn());
                rows += row;
            } catch (Exception e) {
                log.error("入仓ID为" + device.getId() + "的设备失败", e);
                throw e;
            }
        }
        return rows;
    }

    // 入仓
    private int in(DeviceVO device) {
        if (DeviceStatus.STORAGE.getCode().equals(device.getStatus())) {
            return 1;
        }
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(device.getId() == null, "设备ID不能为空");
        ServiceUtil.assertion(!DeviceStatus.canIn().contains(device.getStatus()), "设备%s当前状态不允许入仓", device.getSn());

        // 更新设备状态
        Device data = new Device();
        data.setStatus(DeviceStatus.STORAGE.getCode());
        DeviceQuery query = new DeviceQuery();
        query.setId(device.getId());
        query.setStatusList(DeviceStatus.canIn());
        return deviceMapper.updateByQuery(data, query);
    }

    @Override
    public int out(List<DeviceVO> deviceList) {
        if (CollectionUtils.isEmptyElement(deviceList)) {
            return 0;
        }
        int rows = 0;
        for (DeviceVO device : deviceList) {
            if (device == null) {
                continue;
            }
            try {
                int row = this.out(device);
                ServiceUtil.assertion(row != 1, "设备%s出仓失败，请刷新后重试", device.getSn());
                rows += row;
            } catch (Exception e) {
                log.error("出仓ID为" + device.getId() + "的设备失败", e);
                throw e;
            }
        }
        return rows;
    }

    // 出仓
    private int out(DeviceVO device) {
        if (DeviceStatus.AVAILABLE.getCode().equals(device.getStatus())) {
            return 1;
        }
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(device.getId() == null, "设备ID不能为空");
        ServiceUtil.assertion(!DeviceStatus.canOut().contains(device.getStatus()), "设备%s当前状态不允许出仓", device.getSn());

        // 更新设备状态
        Device data = new Device();
        data.setStatus(DeviceStatus.AVAILABLE.getCode());
        DeviceQuery query = new DeviceQuery();
        query.setId(device.getId());
        query.setStatusList(DeviceStatus.canOut());
        return deviceMapper.updateByQuery(data, query);
    }

    @Override
    public int disable(List<DeviceVO> deviceList) {
        if (CollectionUtils.isEmptyElement(deviceList)) {
            return 0;
        }
        int rows = 0;
        for (DeviceVO device : deviceList) {
            if (device == null) {
                continue;
            }
            try {
                int row = this.disable(device);
                ServiceUtil.assertion(row != 1, "设备%s禁用失败，请刷新后重试", device.getSn());
                rows += row;
            } catch (Exception e) {
                log.error("禁用ID为" + device.getId() + "的设备失败", e);
                throw e;
            }
        }
        return rows;
    }

    // 禁用
    private int disable(DeviceVO device) {
        if (DeviceStatus.DISABLED.getCode().equals(device.getStatus())) {
            return 1;
        }
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(device.getId() == null, "设备ID不能为空");
        ServiceUtil.assertion(!DeviceStatus.canDisable().contains(device.getStatus()), "设备%s当前状态不允许禁用", device.getSn());

        // 更新设备状态
        Device data = new Device();
        data.setStatus(DeviceStatus.DISABLED.getCode());
        DeviceQuery query = new DeviceQuery();
        query.setId(device.getId());
        query.setStatusList(DeviceStatus.canDisable());
        return deviceMapper.updateByQuery(data, query);
    }

    @Override
    public int enable(List<DeviceVO> deviceList) {
        if (CollectionUtils.isEmptyElement(deviceList)) {
            return 0;
        }
        int rows = 0;
        for (DeviceVO device : deviceList) {
            if (device == null) {
                continue;
            }
            try {
                int row = this.enable(device);
                ServiceUtil.assertion(row != 1, "设备%s禁用失败，请刷新后重试", device.getSn());
                rows += row;
            } catch (Exception e) {
                log.error("禁用ID为" + device.getId() + "的设备失败", e);
                throw e;
            }
        }
        return rows;
    }

    // 启用
    private int enable(DeviceVO device) {
        if (DeviceStatus.AVAILABLE.getCode().equals(device.getStatus())) {
            return 1;
        }
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(device.getId() == null, "设备ID不能为空");
        ServiceUtil.assertion(!DeviceStatus.canEnable().contains(device.getStatus()), "设备%s当前状态不允许启用", device.getSn());

        // 更新设备状态
        Device data = new Device();
        data.setStatus(DeviceStatus.AVAILABLE.getCode());
        DeviceQuery query = new DeviceQuery();
        query.setId(device.getId());
        query.setStatusList(DeviceStatus.canEnable());
        return deviceMapper.updateByQuery(data, query);
    }   

    @Override
    public DeviceVO selectDeviceBySn(String sn, boolean scope) {
        if (StringUtils.isBlank(sn)) {
            return null;
        }
        DeviceQuery query = new DeviceQuery();
        query.setEqSn(sn);
        query.setScope(scope);
        return this.selectOne(query);
    }

    @Override
    public DeviceVO selectSimpleByMacForLocationLog(String mac) {
        if (StringUtils.isBlank(mac)) {
            return null;
        }
        DeviceQuery query = new DeviceQuery();
        query.setEqMac(mac);
        return deviceMapper.selectSimpleForLocationLog(query);
    }

    @Override
    public int updateCurrentOrderId(Long id, Long orderId) {
        if (id == null || orderId == null) {
            return 0;
        }
        Device data = new Device();
        data.setOrderId(orderId);
        data.setId(id);
        return deviceMapper.updateDevice(data);
    }
}
