package com.ruoyi.bst.order.service.impl;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.github.pagehelper.PageHelper;
import com.ruoyi.bst.app.service.AppService;
import com.ruoyi.bst.bonus.domain.BonusVO;
import com.ruoyi.bst.bonus.service.BonusConverter;
import com.ruoyi.bst.channel.service.ChannelService;
import com.ruoyi.bst.device.service.DeviceService;
import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.domain.bo.OrderCreateBO;
import com.ruoyi.bst.order.domain.bo.OrderPayBO;
import com.ruoyi.bst.order.domain.dto.OrderCreateDTO;
import com.ruoyi.bst.order.mapper.OrderMapper;
import com.ruoyi.bst.order.service.OrderConverter;
import com.ruoyi.bst.order.service.OrderService;
import com.ruoyi.bst.order.service.OrderValidator;
import com.ruoyi.bst.pay.domain.vo.DoPayVO;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;

/**
 * 订单Service业务层处理
 *
 * @author ruoyi
 * @date 2025-03-24
 */
@Service
public class OrderServiceImpl implements OrderService
{
    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private OrderConverter orderConverter;

    @Autowired
    private OrderValidator orderValidator;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private ChannelService channelService;

    @Autowired
    private AppService appService;

    @Autowired
    private BonusConverter bonusConverter;

    /**
     * 查询订单
     *
     * @param id 订单主键
     * @return 订单
     */
    @Override
    public OrderVO selectOrderById(Long id, boolean scope)
    {
        if (id == null) {
            return null;
        }
        OrderQuery query = new OrderQuery();
        query.setId(id);
        query.setScope(scope);
        return this.selectOne(query);
    }

    @Override
    public OrderVO selectOne(OrderQuery query) {
        PageHelper.startPage(1, 1);
        List<OrderVO> list = this.selectOrderList(query);
        return CollectionUtils.firstElement(list);
    }

    /**
     * 查询订单列表
     *
     * @param order 订单
     * @return 订单
     */
    @Override
    public List<OrderVO> selectOrderList(OrderQuery order)
    {
        return orderMapper.selectOrderList(order);
    }

    /**
     * 新增订单
     *
     * @param order 订单
     * @return 结果
     */
    @Override
    public int insertOrder(Order order)
    {
        order.setCreateTime(DateUtils.getNowDate());
        order.setNo(String.valueOf(SnowFlakeUtil.newId()));
        return orderMapper.insertOrder(order);
    }

    /**
     * 修改订单
     *
     * @param order 订单
     * @return 结果
     */
    @Override
    public int updateOrder(Order order)
    {
        return orderMapper.updateOrder(order);
    }

    /**
     * 批量删除订单
     *
     * @param ids 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderByIds(Long[] ids)
    {
        return orderMapper.deleteOrderByIds(ids);
    }

    /**
     * 删除订单信息
     *
     * @param id 订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderById(Long id)
    {
        return orderMapper.deleteOrderById(id);
    }

    @Override
    public OrderVO createOrder(OrderCreateDTO dto) {
        ServiceUtil.assertion(dto == null || dto.getDeviceId() == null, "参数错误");

        // 加锁，防止设备重复下单
        Long lockKey = dto.getDeviceId();
        boolean lock = redisLock.lock(RedisLockKey.ORDER_CREATE, lockKey);
        ServiceUtil.assertion(!lock, "当前设备使用的人太多了，请稍后再试");
        try {
            // 转为BO对象
            OrderCreateBO bo = orderConverter.toCreateBO(dto);
            // 校验参数
            orderValidator.validate(bo);
    
            return transactionTemplate.execute(status -> {
                // 创建订单
                Order order = bo.getOrder();
                int rows = this.insertOrder(order);
                if (rows != 1) {
                    return null;
                }
                
                // 查询订单
                OrderVO vo = this.selectOrderByNo(order.getNo());

                // 更新设备当前订单ID
                int updateDevice = deviceService.updateCurrentOrderId(bo.getDevice().getId(), vo.getId());
                ServiceUtil.assertion(updateDevice != 1, "更新设备当前订单ID失败");

                // 加入延时队列，超时取消
                this.cancelWhenExpired(order);

                return vo;
            });
        } finally {
            redisLock.unlock(RedisLockKey.ORDER_CREATE, lockKey);
        }
    }

    @Override
    public OrderVO selectOrderByNo(String no, boolean scope) {
        if (StringUtils.isBlank(no)) {
            return null;
        }
        OrderQuery query = new OrderQuery();
        query.setEqNo(no);
        query.setScope(scope);
        return this.selectOne(query);
    }

    // 加入延时队列，超时取消
    private void cancelWhenExpired(Order order) {
        if (order == null || order.getPayExpireTime() == null) {
            return;
        }

        Duration duration = Duration.between(LocalDateTime.now(), order.getPayExpireTime());
        long delay = duration.getSeconds();
        if (delay <= 0) {
            delay = 1;
        }

        scheduledExecutorService.schedule(() -> {
            // 尝试取消订单
            this.cancelOrder(order.getId());
        }, delay, TimeUnit.SECONDS);
    }


    // TODO 取消订单
    @Override
    public int cancelOrder(Long id) {
        if (id == null) {
            return 0;
        }
        // TODO
        return 0;
    }


    @Override
    public DoPayVO pay(OrderVO order, Long appId, Long channelId) {
        ServiceUtil.assertion(order == null || appId == null || channelId == null, "参数错误");

        OrderPayBO bo = orderConverter.toPayBO(order, appId, channelId);
        orderValidator.validate(bo);

        // 转为分成列表
        List<BonusVO> bonuseList = bonusConverter.toBonusList(bo);
        
        
    }
}
