package com.ruoyi.system.dept.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.TreeSelect;
import com.ruoyi.common.core.domain.entity.Dept;
import com.ruoyi.common.core.domain.entity.Role;
import com.ruoyi.common.core.domain.entity.User;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.dept.domain.DeptQuery;
import com.ruoyi.system.dept.domain.DeptVO;
import com.ruoyi.system.dept.domain.vo.DeptNameVO;
import com.ruoyi.system.dept.mapper.DeptMapper;
import com.ruoyi.system.dept.service.DeptService;
import com.ruoyi.system.role.mapper.RoleMapper;

/**
 * 部门管理 服务实现
 *
 * @author ruoyi
 */
@Service
public class SysDeptServiceImpl implements DeptService
{
    @Autowired
    private DeptMapper deptMapper;

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询部门管理数据
     *
     * @param query 部门信息
     * @return 部门信息集合
     */
    @Override
    public List<DeptVO> selectDeptList(DeptQuery query)
    {
        return deptMapper.selectDeptList(query);
    }

    /**
     * 查询部门树结构信息
     *
     * @param query 部门信息
     * @return 部门树信息集合
     */
    @Override
    public List<TreeSelect> selectDeptTreeList(DeptQuery query)
    {
        List<DeptVO> depts = this.selectDeptList(query);
        return buildDeptTreeSelect(depts);
    }

    /**
     * 构建前端所需要树结构
     *
     * @param depts 部门列表
     * @return 树结构列表
     */
    @Override
    public List<DeptVO> buildDeptTree(List<DeptVO> depts)
    {
        List<DeptVO> returnList = new ArrayList<>();
        List<Long> tempList = depts.stream().map(Dept::getDeptId).collect(Collectors.toList());
        for (DeptVO dept : depts) {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(dept.getParentId())) {
                recursionFn(depts, dept);
                returnList.add(dept);
            }
        }
        if (returnList.isEmpty()) {
            returnList = depts;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param depts 部门列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildDeptTreeSelect(List<DeptVO> depts)
    {
        List<DeptVO> deptTrees = buildDeptTree(depts);
        return deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId 角色ID
     * @return 选中部门列表
     */
    @Override
    public List<Long> selectDeptListByRoleId(Long roleId)
    {
        Role role = roleMapper.selectRoleById(roleId);
        return deptMapper.selectDeptListByRoleId(roleId, role.isDeptCheckStrictly());
    }

    /**
     * 根据部门ID查询信息
     *
     * @param deptId 部门ID
     * @param scope
     * @return 部门信息
     */
    @Override
    public Dept selectDeptById(Long deptId, boolean scope)
    {
        return deptMapper.selectDeptById(deptId, scope);
    }

    /**
     * 根据ID查询所有子部门（正常状态）
     *
     * @param deptId 部门ID
     * @return 子部门数
     */
    @Override
    public int selectNormalChildrenDeptById(Long deptId)
    {
        return deptMapper.selectNormalChildrenDeptById(deptId);
    }

    /**
     * 是否存在子节点
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public boolean hasChildByDeptId(Long deptId)
    {
        int result = deptMapper.hasChildByDeptId(deptId);
        return result > 0;
    }

    /**
     * 查询部门是否存在用户
     *
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistUser(Long deptId)
    {
        int result = deptMapper.checkDeptExistUser(deptId);
        return result > 0;
    }

    /**
     * 校验部门名称是否唯一
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public boolean checkDeptNameUnique(Dept dept)
    {
        Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        Dept info = deptMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
        if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验部门是否有数据权限
     *
     * @param deptId 部门id
     */
    @Override
    public void checkDeptDataScope(Long deptId)
    {
        if (!User.isAdmin(SecurityUtils.getUserId()) && StringUtils.isNotNull(deptId))
        {
            DeptQuery dept = new DeptQuery();
            dept.setDeptId(deptId);
            List<DeptVO> depts = this.selectDeptList(dept);
            if (StringUtils.isEmpty(depts))
            {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }

    /**
     * 新增保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public int insertDept(Dept dept)
    {
        Dept info = deptMapper.selectDeptById(dept.getParentId(), false);
        // 如果父节点不为正常状态,则不允许新增子节点
        if (!UserConstants.DEPT_NORMAL.equals(info.getStatus()))
        {
            throw new ServiceException("部门停用，不允许新增");
        }
        dept.setAncestors(String.valueOf(dept.getParentId()));

        Integer result = transactionTemplate.execute(status -> {
            int insert = deptMapper.insertDept(dept);
            if (insert > 0) {
                clearCache();
            }
            return insert;
        });

        return result == null ? 0 : result;
    }

    /**
     * 修改保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public int updateDept(Dept dept)
    {
        Dept newParentDept = deptMapper.selectDeptById(dept.getParentId(), false);
        Dept oldDept = deptMapper.selectDeptById(dept.getDeptId(), false);

        Integer result = transactionTemplate.execute(status -> {
            if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept))
            {
                String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
                String oldAncestors = oldDept.getAncestors();
                dept.setAncestors(newAncestors);
                updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
            }
            int update = deptMapper.updateDept(dept);
            if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors())
                    && !StringUtils.equals("0", dept.getAncestors()))
            {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
                updateParentDeptStatusNormal(dept);
            }

            clearCache();

            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 修改该部门的父级部门状态
     *
     * @param dept 当前部门
     */
    private void updateParentDeptStatusNormal(Dept dept)
    {
        String ancestors = dept.getAncestors();
        Long[] deptIds = Convert.toLongArray(ancestors);
        deptMapper.updateDeptStatusNormal(deptIds);
    }

    /**
     * 修改子元素关系
     *
     * @param deptId 被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    public void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors)
    {
        List<DeptVO> children = deptMapper.selectChildrenDeptById(deptId);
        for (Dept child : children)
        {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (children.size() > 0)
        {
            deptMapper.updateDeptChildren(children);
        }
    }

    /**
     * 删除部门管理信息
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public int deleteDeptById(Long deptId)
    {
        Integer result = transactionTemplate.execute(status -> {
            int delete = deptMapper.deleteDeptById(deptId);
            if (delete > 0) {
                clearCache();
            }
            return delete;
        });

        return result == null ? 0 : result;
    }

    @Override
    public List<DeptVO> selectDeptListByNames(List<String> names) {
        if (CollectionUtils.isEmptyElement(names)) {
            return Collections.emptyList();
        }
        DeptQuery query = new DeptQuery();
        query.setDeptNames(names);
        return this.selectDeptList(query);
    }


    @Override
    public List<DeptNameVO> selectAllDeptNameList() {
        List<DeptNameVO> deptNameList = redisCache.getCacheList(CacheConstants.DEPT_NAME_LIST);
        if (CollectionUtils.isEmptyElement(deptNameList)) {
            deptNameList = deptMapper.selectDeptNameList(new DeptQuery());
            if (CollectionUtils.isNotEmptyElement(deptNameList)) {
                redisCache.setCacheList(CacheConstants.DEPT_NAME_LIST, deptNameList);
            }
        }
        return deptNameList;
    }

    private void clearCache() {
        redisCache.deleteObject(CacheConstants.DEPT_NAME_LIST);
    }

    /**
     * 递归列表
     */
    private void recursionFn(List<? extends Dept> list, Dept t)
    {
        // 得到子节点列表
        List<? extends Dept> childList = getChildList(list, t);
        t.setChildren(childList);
        for (Dept tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<Dept> getChildList(List<? extends Dept> list, Dept t)
    {
        List<Dept> tlist = new ArrayList<Dept>();
        Iterator<? extends Dept> it = list.iterator();
        while (it.hasNext())
        {
            Dept n = (Dept) it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getDeptId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<? extends Dept> list, Dept t)
    {
        return getChildList(list, t).size() > 0;
    }
}
